/**	$Revision: 1.10 $
	Last updated: $Date: 2006/02/26 00:29:05 $

	Copyright (C) 2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "EditCurrencyForm.h"
#include "CurrencyListForm.h"
#include "CoinsPref.h"

EditCurrencyForm::EditCurrencyForm() : Form (EDIT_CURRENCY_FORM)
{}

Boolean EditCurrencyForm::checkData()
{
	bool error = false;
	
	//check for duplicate code
	if(!_record_is_new) {
		if(currency_db->checkForExistingCode (GetTextPtr (EDIT_CURRENCY_CODE), currency_db->getSelectedUid())) 
			error = true;
	} else {
		if(currency_db->checkForExistingCode (GetTextPtr (EDIT_CURRENCY_CODE))) 
			error = true;
	}
	 
	if(error) {
		alert_res (GENERIC_INFO, DUPLICATE_CODE_STR);
		return false;		
	}
	
	//check for empty code
	if(StrLen (GetTextPtr (EDIT_CURRENCY_CODE)) == 0) {
		alert_res (GENERIC_INFO, EMPTY_CODE_STR);
		return false;		
	}	
	//should check rate format
	if (GetTextPtr (EDIT_CURRENCY_RATE) == NULL) {
		alert_res (GENERIC_INFO, RATE_IS_0_STR);
		return false;
	}
	
	FlpCompDouble rate;
	FlpBufferAToF (&rate.fd, GetTextPtr (EDIT_CURRENCY_RATE));
	if (FlpIsZero (rate.d)) {
		alert_res (GENERIC_INFO, RATE_IS_0_STR);
		return false;
	}
	_currency.exchange_rate = rate.d;
	return true; 
}

void EditCurrencyForm::saveData()
{
	_currency.description = GetTextPtr (EDIT_CURRENCY_DESC);
	_currency.code = GetTextPtr (EDIT_CURRENCY_CODE);
	//rate is saved in checkData

	if (!_currency.description) _currency.description = "";
	if (!_record_is_new) {
		currency_db->packRecord (&_currency, _currency_handle);
		DmInsertionSort (currency_db->getRef(), (DmComparF *) currency_db->compareRecords,0);
	} else {		
		currency_db->newRecord (&_currency);
	}

}

Boolean EditCurrencyForm::handleOpenEvent()
{
	FrmDrawForm (frm);
	if (!_record_is_new) {
		frmtitle_res (frm, CUR_EDIT_STR);
		FrmShowObject(frm, getObjectIndex(EDIT_CURRENCY_DEF));
		FrmShowObject(frm, getObjectIndex(EDIT_CURRENCY_DELETE));
		_currency_handle = currency_db->getUnpackedRecord (&_currency);
		SetFieldTextFromStr (EDIT_CURRENCY_CODE, _currency.code);
		SetFieldTextFromStr (EDIT_CURRENCY_DESC, _currency.description);
		FlpCompDouble rate;
		rate.d = _currency.exchange_rate;
		FlpFToA (rate.fd, _rate_buffer);
		SetFieldTextFromStr (EDIT_CURRENCY_RATE, _rate_buffer);
	} else {
		frmtitle_res(frm, CUR_NEW_STR);
		_currency.code = "";
		_currency.description = "";
		_currency.exchange_rate = 0.0;		
	}
	return true;
}
Boolean EditCurrencyForm::handleCtlSelectEvent(UInt16 control_id)
{
	Boolean handled = true;

	switch(control_id) {
		case EDIT_CURRENCY_DELETE: {	
			if (alert_res (DELETE_ALERT, CURRENCY_STR)) break;
			if(!currency_db->deleteRecord(currency_db->getSelectedRecord())) break;
			
			FrmUpdateForm(caller_form_id, 0);
			/* force call to update event as the previous line does not seem to have any effect. Attempts to debug with gdb have
			   failed due to gdb's incapability to deal with code sections - running code step by step simply doesn't work */
			CurrencyListForm::instance()->handleUpdateEvent (0);
			FrmReturnToForm(0);
			break;	
			}
		case EDIT_CURRENCY_OK:
			if (checkData()) {
				saveData();
				FrmUpdateForm(caller_form_id, 0);
				FrmReturnToForm(0);
			}
			break;
		case EDIT_CURRENCY_CANCEL:
			FrmReturnToForm(0);
			break;
		case EDIT_CURRENCY_DEF:
			currency_db->newDefaultCurrency (currency_db->getSelectedUid());
			preferences.get()->def_currency = currency_db->getSelectedUid();
			FrmUpdateForm(caller_form_id, 0);
			FrmReturnToForm(0);
			break;
		default:
			handled = false;
	}
	return handled;
}
