/**	$Revision: 1.24 $
	Last updated: $Date: 2006/03/09 18:58:39 $

	Copyright (C) 2000-2002 Vlad Mereuta <dizzy@users.sourceforge.net>

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA */

#include "EditSplitForm.h"
#include "SplitsForm.h"
#include "ChooseAccountForm.h"
#include "FreeCoinsRsc.h"
#include "AccountsDB.h"
#include "CalcForm.h"


EditSplitForm::EditSplitForm() : Form (EDIT_SPLIT_FORM)
{}

Boolean EditSplitForm::checkData(void)
{
	Char *s;
	if (!acc_uid) {
		alert_res (GENERIC_ERROR, CHOOSE_ACC_STR);
		return false;
	}
	s = (Char*)GetTextPtr(EDIT_SPLIT_AMOUNT);
	amount = String2Pence(s)*(is_debit?1:-1);
	if (!validNumberStr(s, false) || amount==0) {
		alert_res (GENERIC_ERROR, POZ_NUMBER_STR);
		return false;
	}
	return true;
}

void EditSplitForm::saveData(void)
{
	if (split_index==255) {
		split_index = SplitsForm::instance()->getCount();
		SplitsForm::instance()->setCount(split_index + 1);
	}
	// if the user changes the split for the currently selected account,
	// change the selected account accordingly. 
	if (SplitsForm::instance()->getAccount(split_index)==accounts_db->getSelectedUid())
		accounts_db->selectRecord(acc_uid);
	SplitsForm::instance()->setAccount(split_index, acc_uid);
	SplitsForm::instance()->setAmount(split_index, amount);
}

Boolean EditSplitForm::handleOpenEvent()
{
	Char* s;
	
	if (split_index!=255) {
		acc_uid = SplitsForm::instance()->getAccount(split_index);
		amount = SplitsForm::instance()->getAmount(split_index);
		// don't allow the user to delete the entry for the current register account
		if (acc_uid != accounts_db->getSelectedUid())
			FrmShowObject(frm, FrmGetObjectIndex(frm, EDIT_SPLIT_DELETE));
		frmtitle_res (frm, EDIT_SPLIT_STR);
	} else {
		amount = 0;
		acc_uid = 0;
		frmtitle_res (frm, NEW_SPLIT_STR);
	}

	os35_drawForm(frm);

	is_debit = (amount>0);
	s = Pence2String(is_debit?amount:-amount);
	SetFieldTextFromStr(EDIT_SPLIT_AMOUNT, s);
	MemPtrFree(s);

	FrmUpdateForm(caller_form_id, 0);

	return true;
}
Boolean EditSplitForm::handleCtlSelectEvent(UInt16 control_id)
{
	Boolean handled = true;

	switch(control_id) {
	case EDIT_SPLIT_OK:
		if (checkData()) {
			saveData();
			FrmUpdateForm(caller_form_id, 0);
			FrmReturnToForm(0);
		}
		break;
	case EDIT_SPLIT_CANCEL:
		FrmUpdateForm (caller_form_id,0);
		FrmReturnToForm(0);
		break;
	case EDIT_SPLIT_DELETE: {
		if (alert_res (DELETE_ALERT, SPLIT_STR)) break;
		SplitsForm::instance()->deleteSplit(split_index);		

		//the line below doesn't seem to have any effect
		FrmUpdateForm(caller_form_id, 0);
		//EvtAddEventToQueue (&e);		
		//force update:

		//as the update event gets lost we need to force the update; we use code 2 so that the arrows aren't diplsayed 
		//(as current form is not correct, trying to display the arrows would produce an error)
		SplitsForm::instance()->handleUpdateEvent (2);
		FrmReturnToForm(0);
		break;	
	}
	case EDIT_SPLIT_DEBIT:
		is_debit = true;
		break;
	case EDIT_SPLIT_CREDIT:
		is_debit = false;
		break;
	case EDIT_SPLIT_CHOOSE_ACC:
		ChooseAccountForm::instance()->allow_no_selection = false;
		ChooseAccountForm::instance()->chosen = false;
		ChooseAccountForm::instance()->chosen_account = acc_uid;
		popup(ChooseAccountForm::instance());
		break;
		
	case EDIT_SPLIT_CALC:
		CalculatorForm::instance()->amount = String2Pence((Char*)GetTextPtr(EDIT_SPLIT_AMOUNT)) *	(is_debit?1:-1);
		if (CalculatorForm::instance()->amount < 0) {
			CalculatorForm::instance()->amount *= -1;
			CalculatorForm::instance()->sign = -1;
		} else CalculatorForm::instance()->sign = 1;
		popup(CalculatorForm::instance());

	default:
		handled = false;
	}
	return handled;
}

Boolean EditSplitForm::handleUpdateEvent(UInt16 update_code)
{
	ControlPtr ctl;
	Char *s;
	accountStructure  as;
	UInt8 type;

	if (ChooseAccountForm::instance()->chosen) {
		ChooseAccountForm::instance()->chosen = false;
		if (acc_uid != ChooseAccountForm::instance()->chosen_account) {
			for (UInt8 i=0; i<SplitsForm::instance()->getCount(); i++) {
				if (SplitsForm::instance()->getAccount(i)==ChooseAccountForm::instance()->chosen_account) {
					ChooseAccountForm::instance()->chosen_account = 0;
					alert_res (GENERIC_ERROR, ACCOUNT_PART_STR);
					break;
				}
			}
			if (ChooseAccountForm::instance()->chosen_account)
				acc_uid = ChooseAccountForm::instance()->chosen_account;
		}
	}

	//set account name and debit/credit button labels
	if (acc_uid) {
		accounts_db->getUnpackedRecord(acc_uid, &as);
		type = as.type;
		SysStringByIndex(ACC_DEBIT_DESC, (UInt8)type, debit_label, 32);
		SysStringByIndex(ACC_CREDIT_DESC,(UInt8)type,credit_label, 32);
		accounts_db->getFullNameTruncLeftToFit(acc_uid, 120, acc_name);
	} else {
		strcopy_res (debit_label, DEBIT_LABEL_STR);
		strcopy_res (credit_label, CREDIT_LABEL_STR);
		strcopy_res (acc_name, NOT_SET_STR);
	}
	
	ctl = (ControlPtr) GetObjectPtr(EDIT_SPLIT_DEBIT);
	CtlSetLabel(ctl, debit_label);
	ctl = (ControlPtr) GetObjectPtr(EDIT_SPLIT_CREDIT);
	CtlSetLabel(ctl, credit_label);
	ctl = (ControlPtr) GetObjectPtr(EDIT_SPLIT_CHOOSE_ACC);
	CtlSetLabel(ctl, acc_name);

	if(update_code == 0 || update_code == frmRedrawUpdateCode)
		os35_drawForm(frm);

	if(update_code == CalculatorForm::instance()->update_code) {
		amount = CalculatorForm::instance()->amount *	CalculatorForm::instance()->sign;
		is_debit = (amount>0);
		s = Pence2String(is_debit?amount:-amount);
		SetFieldTextFromStr(EDIT_SPLIT_AMOUNT, s);
		MemPtrFree(s);
	}

	// set debit/credit button
	FrmSetControlGroupSelection(frm, 1,is_debit?EDIT_SPLIT_DEBIT:EDIT_SPLIT_CREDIT);

	return true;
}
